using System;
using System.Collections.Generic;
using System.Text;
using AoLib.Utils;

namespace VhaBot.Plugins
{
    public class VhPrivateChannel : PluginBase
    {
        private bool _locked = false;
        private string _lockedBy = string.Empty;
        private bool _announce = true;

        public VhPrivateChannel()
        {
            this.Name = "Private Channel Manager";
            this.InternalName = "vhPrivateChannelManager";
            this.Author = "Vhab";
            this.Version = 100;
            this.DefaultState = PluginState.Installed;
            this.Commands = new Command[] {
                new Command("join", true, UserLevel.Member),
                new Command("leave", true, UserLevel.Guest),
                new Command("invite", true, UserLevel.Leader, UserLevel.Leader, UserLevel.Member),
                new Command("kick", true, UserLevel.Leader, UserLevel.Leader, UserLevel.Member),
                new Command("lock", true, UserLevel.Leader),
                new Command("unlock", true, UserLevel.Leader),

                new Command("guestlist", true, UserLevel.Leader),
                new Command("guestlist add", true, UserLevel.Leader),
                new Command("guestlist remove", true, UserLevel.Leader),
            };
        }

        public override void OnLoad(BotShell bot)
        {
            bot.Configuration.Register(ConfigType.Boolean, this.InternalName, "announce", "Announce joins and leaves", this._announce);
            bot.Events.UserJoinChannelEvent += new UserJoinChannelHandler(Events_UserJoinChannelEvent);
            bot.Events.UserLeaveChannelEvent += new UserLeaveChannelHandler(Events_UserLeaveChannelEvent);
            bot.Events.UserLogonEvent += new UserLogonHandler(Events_UserLogonEvent);
            bot.Events.ConfigurationChangedEvent += new ConfigurationChangedHandler(Events_ConfigurationChangedEvent);

            this._announce = bot.Configuration.GetBoolean(this.InternalName, "announce", this._announce);
        }

        void Events_ConfigurationChangedEvent(BotShell bot, ConfigurationChangedArgs e)
        {
            if (e.Section == this.InternalName)
            {
                switch (e.Key)
                {
                    case "announce":
                        this._announce = (bool)e.Value;
                        break;
                }
            }
        }

        public override void OnUnload(BotShell bot)
        {
            bot.Events.UserJoinChannelEvent -= new UserJoinChannelHandler(Events_UserJoinChannelEvent);
            bot.Events.UserLeaveChannelEvent -= new UserLeaveChannelHandler(Events_UserLeaveChannelEvent);
            bot.Events.UserLogonEvent -= new UserLogonHandler(Events_UserLogonEvent);
            bot.Events.ConfigurationChangedEvent -= new ConfigurationChangedHandler(Events_ConfigurationChangedEvent);
        }

        private void Events_UserLeaveChannelEvent(BotShell bot, UserLeaveChannelArgs e)
        {
            if (this._announce)
            {
                string message = bot.ColorHeader + e.Sender + bot.ColorHighlight + " has left the private channel";
                bot.SendPrivateChannelMessage(message);
                /*if (bot.Config.GetString("simpleirc", "mode", "none") == "both")
                    bot.SendOrganizationMessage(message);*/
            }
        }

        private void Events_UserJoinChannelEvent(BotShell bot, UserJoinChannelArgs e)
        {
            if (this._announce)
            {
                string message;
                if (e.SenderWhois != null)
                    message = bot.ColorHeader + Format.Whois(e.SenderWhois, FormatStyle.Medium) + bot.ColorHighlight + " has joined the private channel";
                else
                    message = bot.ColorHeader + e.Sender + bot.ColorHighlight + " has joined the private channel";
                bot.SendPrivateChannelMessage(message);
                /*if (bot.Config.GetString("simpleirc", "mode", "none") == "both")
                    bot.SendOrganizationMessage(message);*/
            }
        }

        private void Events_UserLogonEvent(BotShell bot, UserLogonArgs e)
        {
            if (e.First) return;
            if (e.Sections.Contains("guestlist"))
            {
                bot.SendPrivateMessage(e.SenderID, bot.ColorHighlight + "You have been invited to the Private Channel because you're on this bot's guestlist. To remove yourself from the guestlist use: " + HTML.CreateColorString(bot.ColorHeaderHex, "/tell " + bot.Character + " guestlist remove"));
                bot.PrivateChannel.Invite(e.SenderID);
            }
        }

        public override void OnCommand(BotShell bot, CommandArgs e)
        {
            if (e.FromSlave && e.Type == CommandType.PrivateChannel)
                return;
            int messageID;

            switch (e.Command)
            {
                case "lock":
                    if (this._locked)
                    {
                        bot.SendReply(e, "The private channel already is locked");
                        return;
                    }
                    this._lockedBy = e.Sender;
                    this._locked = true;
                    string lockMessage = bot.ColorHeader + e.Sender + bot.ColorHighlight + " has locked the private channel";
                    bot.SendPrivateChannelMessage(lockMessage);
                    /*if (bot.Config.GetString("simpleirc", "mode", "none") == "both")
                        bot.SendOrganizationMessage(lockMessage);*/
                    break;
                case "unlock":
                    if (!this._locked)
                    {
                        bot.SendReply(e, "The private channel already is unlocked");
                        return;
                    }
                    this._locked = false;
                    string unlockMessage = bot.ColorHeader + e.Sender + bot.ColorHighlight + " has unlocked the private channel";
                    bot.SendPrivateChannelMessage(unlockMessage);
                    /*if (bot.Config.GetString("simpleirc", "mode", "none") == "both")
                        bot.SendOrganizationMessage(unlockMessage);*/
                    break;
                case "join":
                    if (bot.PrivateChannel.IsOn(e.SenderID))
                    {
                        bot.SendReply(e, "You already are on the private channel");
                        return;
                    }
                    if (this._locked && bot.Users.GetUser(e.Sender) < UserLevel.Leader)
                    {
                        bot.SendReply(e, "The private channel is currently locked by " + bot.ColorHeader + this._lockedBy);
                        return;
                    }
                    bot.SendReply(e, "Inviting you to the private channel");
                    bot.PrivateChannel.Invite(e.SenderID);
                    break;
                case "leave":
                    if (!bot.PrivateChannel.IsOn(e.SenderID))
                    {
                        bot.SendReply(e, "You're not on the private channel");
                        return;
                    }
                    if (e.Type != CommandType.PrivateChannel)
                        bot.SendReply(e, "Kicking you from the private channel");

                    // Remove a user from the vhabot raid system before letting him leave
                    if (bot.Plugins.IsLoaded("raidcore"))
                        bot.SendPluginMessage(this.InternalName, "raidcore", "RemoveRaider", out messageID, e.Sender);

                    bot.PrivateChannel.Kick(e.SenderID);
                    break;
                case "invite":
                    if (e.Args.Length < 1)
                    {
                        bot.SendReply(e, "Correct Usage: invite [username]");
                        break;
                    }
                    if (bot.GetUserID(e.Args[0]) < 1)
                    {
                        bot.SendReply(e, "No such user: " + HTML.CreateColorString(bot.ColorHeaderHex, e.Args[0]));
                        break;
                    }
                    if (bot.PrivateChannel.IsOn(e.Args[0]))
                    {
                        bot.SendReply(e, HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " already is on the private channel");
                        break;
                    }
                    if (this._locked && bot.Users.GetUser(e.Sender) < UserLevel.Leader)
                    {
                        bot.SendReply(e, "The private channel is currently locked by " + bot.ColorHeader + this._lockedBy);
                        return;
                    }
                    bot.SendReply(e, "Inviting " + HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " to the private channel");
                    bot.PrivateChannel.Invite(e.Args[0]);
                    break;
                case "kick":
                    if (e.Args.Length < 1)
                    {
                        bot.SendReply(e, "Correct Usage: kick [username]");
                        break;
                    }
                    if (e.Args[0].ToLower() == "all")
                    {
                        bot.SendReply(e, "Clearing the private channel");
                        bot.PrivateChannel.KickAll();
                        break;
                    }
                    if (bot.GetUserID(e.Args[0]) < 1)
                    {
                        bot.SendReply(e, "No such user: " + HTML.CreateColorString(bot.ColorHeaderHex, e.Args[0]));
                        break;
                    }
                    if (!bot.PrivateChannel.IsOn(e.Args[0]))
                    {
                        bot.SendReply(e, HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " isn't on the private channel");
                        break;
                    }
                    if (bot.Users.GetMain(e.Args[0]).Equals(bot.Admin, StringComparison.CurrentCultureIgnoreCase))
                    {
                        bot.SendReply(e, "You can't kick the bot owner");
                        break;
                    }
                    if (bot.Users.GetUser(e.Args[0]) > bot.Users.GetUser(e.Sender))
                    {
                        bot.SendReply(e, HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " outranks you");
                        break;
                    }
                    // Remove a user from the vhabot raid system before kicking them
                    if (bot.Plugins.IsLoaded("RaidCore"))
                        bot.SendPluginMessage(this.InternalName, "raidcore", "RemoveRaider", out messageID, e.Args[0]);

                    bot.SendReply(e, "Kicking " + HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " from the private channel");
                    bot.PrivateChannel.Kick(e.Args[0]);
                    break;
                case "guestlist":
                    string[] guestlist = bot.FriendList.List("guestlist");
                    if (guestlist.Length == 0)
                    {
                        bot.SendReply(e, "There are no users on the guestlist");
                        break;
                    }
                    RichTextWindow window = new RichTextWindow(bot);
                    window.AppendTitle("Guestlist");
                    foreach (string guest in guestlist)
                    {
                        window.AppendHighlight(Format.UppercaseFirst(guest));
                        window.AppendNormalStart();
                        window.AppendString(" [");
                        window.AppendBotCommand("Remove", "guestlist remove " + guest);
                        window.AppendString("]");
                        window.AppendColorEnd();
                        window.AppendLineBreak();
                    }
                    bot.SendReply(e, guestlist.Length + " Guests  ", window);
                    break;
                case "guestlist add":
                    if (e.Args.Length < 1)
                    {
                        bot.SendReply(e, "Correct Usage: guestlist add [username]");
                        break;
                    }
                    if (bot.GetUserID(e.Args[0]) < 1)
                    {
                        bot.SendReply(e, "No such user: " + HTML.CreateColorString(bot.ColorHeaderHex, e.Args[0]));
                        break;
                    }
                    if (bot.FriendList.IsFriend("guestlist", e.Args[0]))
                    {
                        bot.SendReply(e, HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " already is on the guestlist");
                        break;
                    }
                    bot.FriendList.Add("guestlist", e.Args[0]);
                    bot.SendReply(e, HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " has been added to the guestlist");
                    break;
                case "guestlist remove":
                    if (e.Args.Length < 1)
                    {
                        if (bot.FriendList.IsFriend("guestlist", e.Sender))
                        {
                            bot.SendReply(e, "You have been removed from the guestlist");
                            bot.FriendList.Remove("guestlist", e.Sender);
                        }
                        else
                        {
                            bot.SendReply(e, "You're not on the guestlist");
                        }
                        break;
                    }
                    if (bot.GetUserID(e.Args[0]) < 1)
                    {
                        bot.SendReply(e, "No such user: " + HTML.CreateColorString(bot.ColorHeaderHex, e.Args[0]));
                        break;
                    }
                    if (!bot.FriendList.IsFriend("guestlist", e.Args[0]))
                    {
                        bot.SendReply(e, HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " is not on the guestlist");
                        break;
                    }
                    bot.FriendList.Remove("guestlist", e.Args[0]);
                    bot.SendReply(e, HTML.CreateColorString(bot.ColorHeaderHex, Format.UppercaseFirst(e.Args[0])) + " has been removed from the guestlist");
                    break;
            }
        }

        public override void OnUnauthorizedCommand(BotShell bot, CommandArgs e)
        {
            switch (e.Command)
            {
                case "join":
                    if (bot.FriendList.IsFriend("guestlist", e.Sender))
                    {
                        e.Authorized = true;
                        this.OnCommand(bot, e);
                    }
                    break;
                case "guestlist remove":
                    if (e.Args.Length == 0)
                    {
                        e.Authorized = true;
                        this.OnCommand(bot, e);
                    }
                    break;
            }
        }

        public override string OnHelp(BotShell bot, string command)
        {
            switch (command)
            {
                case "join":
                    return "Invites you to the bot's private channel.\n" +
                        "Usage: /tell " + bot.Character + " join";
                case "leave":
                    return "Kicks you from the bot's private channel.\n" +
                        "Usage: /tell " + bot.Character + " leave";
                case "invite":
                    return "Invites [username] to the bot's private channel.\n" +
                        "Usage: /tell " + bot.Character + " invite [username]";
                case "kick":
                    return "Kicks [username] from the bot's private channel.\n" +
                        "Usage: /tell " + bot.Character + " kick [username]";
                case "lock":
                    return "Locks the private channel from everyone ranked lower than Leader.\n" +
                        "Usage: /tell " + bot.Character + " lock";
                case "unlock":
                    return "Unlocks the private channel.\n" +
                        "Usage: /tell " + bot.Character + " unlock";
                case "guestlist":
                    return "Displays the current guestlist.\nUsers on the guestlist will be automatically invited to the private channel when they log on.\n" +
                        "Usage: /tell " + bot.Character + " guestlist";
                case "guestlist add":
                    return "Adds [username] to the guestlist.\nUsers on the guestlist will be automatically invited to the private channel when they log on.\n" +
                        "Usage: /tell " + bot.Character + " guestlist add [username]";
                case "guestlist remove":
                    return "Removes [username] from the guestlist.\nIf [username] is not supplied, it will remove you from the guestlist instead.\n" +
                        "Usage: /tell " + bot.Character + " guestlist remove [[username]]";
            }
            return null;
        }
    }
}
